/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : March, 1997                                                    */
/* File Name : Integrator.cc                                             */
/*************************************************************************/
/*************************************************************************/

#include <math.h>
#include "Function.h"
#include "Integrator.h"
#include "../deffunc.h"

Integrator::Integrator(Function *func, double _fr, double _to)
{  myFunc = func;
   from = minimum(_fr,_to);
   to   = maximum(_fr,_to);
}// end of constructor

double Integrator::trapzd(int n)
{
   double x, tnm, sum, del;
   static double ret;
   int it, j;

   if(n==1) 
      ret = 0.5*(to-from)*((*myFunc).getValue(from)+(*myFunc).getValue(to));
   else
     {  for(it=1, j=1; j<n-1;j++) it <<= 1;
	tnm = it;
	del = (to-from)/tnm;
	x = from + 0.5* del;
	for(sum=0.0,j=1;j<=it;j++, x+=del) sum+=(*myFunc).getValue(x);
	ret = 0.5 * ( ret + (to-from) * sum / tnm);
     }

   return ret;   
   
}// end of trapzd()

double Integrator::eval()
{  int j;
   double s, st, ost, os;
   ost = os = 0.0;
   int exit=0;

   if(myFunc!=NULL)
   for(j=1;j<=JMAX && !exit;j++)
   {  st = trapzd(j);
      s = (4.0*st-ost)/3.0;
      if(fabs(s-os)< EPS*fabs(os)) exit=1;
      if(s==0.0 && os == 0.0 && j>6) exit = 1;
      os = s;
      ost = st;
   }

   return (exit?s:0.0);

}// end of eval()


